<?php
// Profiling functions

$rg_prof_main = array();
$rg_prof_state = array();
$rg_prof_tmp = array();

/*
 * This will be called by submodules like git and db
 */
function rg_prof_set($data)
{
	global $rg_prof_state;

	foreach ($data as $k => $v) {
		if (isset($rg_prof_state[$k]))
			$rg_prof_state[$k] += $v;
		else
			$rg_prof_state[$k] = $v;
	}
}

function rg_prof_start($label)
{
	global $rg_prof_tmp;
	global $rg_prof_state;

	$rg_prof_tmp[$label] = $rg_prof_state;
	$rg_prof_tmp[$label]['time_ms'] = sprintf("%u", microtime(TRUE) * 1000);
	$rg_prof_tmp[$label]['mem'] = memory_get_usage();
	$rg_prof_tmp[$label]['real_mem'] = memory_get_usage(TRUE);
}

function rg_prof_end($label)
{
	global $rg_prof_tmp;
	global $rg_prof_state;
	global $rg_prof_main;

	if (!isset($rg_prof_tmp[$label])) {
		rg_internal_error("rg_prof_start was not called for label [$label]!");
		return;
	}

	$start = $rg_prof_tmp[$label];

	$c = $rg_prof_state;
	$c['time_ms'] = sprintf("%u", microtime(TRUE) * 1000);
	$c['runs'] = 1;
	$c['mem'] = memory_get_usage();
	$c['real_mem'] = memory_get_usage(TRUE);

	// we substract what was before start
	foreach ($start as $key => $val)
		$c[$key] -= $val;

	// add to main label
	if (!isset($rg_prof_main[$label])) {
		$rg_prof_main[$label] = $c;
	} else {
		foreach ($c as $key => $val) {
			// we may have items that are present at "end" time
			// and not at "start" time
			if (!isset($rg_prof_main[$label][$key]))
				$rg_prof_main[$label][$key] = $val;
			else
				$rg_prof_main[$label][$key] += $val;
		}
	}

	unset($rg_prof_tmp[$label]);
}

function rg_prof_get()
{
	global $rg_prof_main;

	return $rg_prof_main;
}

function rg_prof_log($cb)
{
	global $rg_prof_main;

	$cb("Profiling:");
	foreach ($rg_prof_main as $label => $per_label) {
		$line = "  " . $label . ":";

		foreach ($per_label as $k => $v) {
			if ($v > 0)
				$line .= " $k=$v";
		}
		$cb($line);
	}
}

function rg_prof_html()
{
	global $rg_prof_main;

	// get all possible variables
	$vars = array();
	foreach ($rg_prof_main as $label => $per_label)
		foreach ($per_label as $k => $v)
			$vars[$k] = 1;

	$ret = "<table>";
	$ret .= "<tr>";
	$ret .= "<td>Label</td>";
	foreach ($vars as $var => $junk)
		$ret .= "<td>" . $var . "</td>";
	$ret .= "</tr>";

	foreach ($rg_prof_main as $label => $per_label) {
		$ret .= "<tr>";

		$ret .= "<td>" . $label . "</td>";

		foreach ($vars as $k => $junk) {
			if (!isset($per_label[$k]))
				$v = 0;
			else
				$v = $per_label[$k];
			$ret .= "<td>" . $v . "</td>";
		}

		$ret .= "</tr>";
	}

	$ret .= "</table>\n";

	return $ret;
}

function rg_prof_text()
{
	global $rg_prof_main;

	$ret = "";

	// get all possible variables
	$vars = array();
	foreach ($rg_prof_main as $label => $per_label)
		foreach ($per_label as $k => $v)
			$vars[$k] = 1;

	$add = "";
	foreach ($rg_prof_main as $label => $per_label) {
		$ret .= $add;
		$add = "\n";

		$ret .= $label . ": ";

		foreach ($vars as $k => $junk) {
			if (!isset($per_label[$k]))
				$v = 0;
			else
				$v = $per_label[$k];
			$ret .= " $k=$v";
		}
	}

	return $ret;
}
?>
