<?php
rg_log("/inc/user/repo-page");

$_repo_page = "";
$repo_more = $more;
rg_log("DEBUG: repo_more: " . rg_array2string($repo_more));

if (rg_user_ok($user) !== TRUE) {
	$_repo_page .= rg_warning("Invalid user!");
	return;
}

if (rg_repo_ok($repo) !== TRUE) {
	$_repo_page .= rg_warning("Invalid repo!");
	return;
}

$ri = rg_repo_info($db, 0, $login_ui['uid'], $repo);
if ($ri['ok'] != 1) {
	$_repo_page .= rg_warning("Internal error!");
	return;
}
if ($ri['exists'] != 1) {
	$_repo_page .= rg_warning("Non existing repo!");
	return;
}

if ($ri['git_dir_done'] == 0) {
	// We will wait a little for the git dir to be done
	// TODO: Should we really wait for this?!
	// We may just consider that the repo is empty. Hm.
	$ev_id = "repo_create-" . $login_ui['uid'] . "-" . $ri['repo_id'] . "-git";
	$timeout = 10; // seconds
	$r = rg_event_signal_daemon($ev_id, $timeout);
	if ($r === FALSE) {
		// Seems we did not get the notification
		$_repo_page .= rg_warning("Internal error!");
		return;
	}
}

// We must not allow access to a private repo (check for fetch)
if (rg_repo_allow($db, $ri, $login_ui, "F") !== TRUE) {
	$_repo_page .= rg_warning("Not existent repo!");
	return;
}

$page_ui = rg_user_info($db, 0, $user, "");
if ($page_ui === FALSE) {
	$_repo_page .= rg_warning("Invalid user!");
	return;
}
// TODO: should we test against $ri?!
$can_admin = (rg_repo_allow($db, $ri, $login_ui, "A") === TRUE) ? 1 : 0;

$_t = array(
	"owner" => $user,
	"url_user" => rg_re_userpage($page_ui),
	"url_repo" => rg_re_repopage($page_ui, $repo),
	"ssh" => rg_re_repo_ssh($organization, $user, $repo),
	"git" => rg_re_repo_git($organization, $user, $repo),
	"can_admin" => $can_admin
	);
$repo_more = array_merge($repo_more, $_t);
$repo_more = array_merge($repo_more, $ri);
$repo_more['default_rights'] = implode(", ", rg_rights_text("repo", $ri['default_rights']));
$repo_more['max_commit_size'] = $ri['max_commit_size'] == 0 ? "unlimited" : rg_1024($ri['max_commit_size']);
$repo_more['max_users'] = $ri['max_users'] == 0 ? "unlimited" : $ri['max_users'];
$repo_more['hints'] = "";

$repo_path = rg_repo_path_by_id($ri['uid'], $ri['repo_id']);
rg_log("repo_path=$repo_path");
putenv("GIT_DIR=$repo_path"); // TODO: this will be removed after all functios will got a path para

$repo_more['repo_body'] = "";
$repo_more['repo_right'] = "";
$repo_more['branches_and_tags'] = "";
$_repo_body = "";
$_repo_right = "";

// build urls list
$urls = array();
if ($rg_ssh_port != 0)
	$urls[]['HTML:url'] = '<a href="' . $repo_more['ssh'] . '">' . $repo_more['ssh'] . '</a>';
if ($rg_git_port != 0)
	$urls[]['HTML:url'] = '<a href="' . $repo_more['git'] . '">' . $repo_more['git'] . '</a>';
$repo_more['HTML:urls'] = rg_template_table("repo/urls", $urls, $repo_more);

// default tab
$_subop = empty($paras) ? "history" : array_shift($paras);

if (strcmp($_subop, "history") == 0) {
	$hist = rg_repo_history_load($db, $ri['repo_id'], 0, 20, 0);
	if ($hist === FALSE)
		$_repo_body .= rg_warning("Cannot load history. Try again later.");
	else
		$_repo_body .= rg_template_table("repo/history", $hist, $repo_more);
} else if (strcmp($_subop, "admin") == 0) {
	include($INC . "/user/repo/admin/admin.php");
	$_repo_body .= $_admin;
} else if (strcmp($_subop, "source") == 0) {
	$_subsubop = empty($paras) ? "" : array_shift($paras);

	$type_ref = rg_git_parse_ref($paras);
	$ref = $type_ref['ref_path'];
	$repo_more = array_merge($repo_more, $type_ref);

	$bt = rg_git_branches_and_tags($repo_path, $repo_more['url_repo'],
		$type_ref['ref_url']);
	//rg_log("DEBUG: repo_more: " . rg_array2string($repo_more));
	$repo_more = array_merge($repo_more, $bt);

	$hints = array();
	if ($rg_ssh_port != 0)
		$hints[]['HTML:hint'] = rg_template("hints/repo/clone_ssh.html", $repo_more);
	if ($rg_git_port != 0)
		$hints[]['HTML:hint'] = rg_template("hints/repo/clone_git.html", $repo_more);
	if (rg_rights_allow($ri['default_rights'], "H")) // anonymous push?
		$hints[]['HTML:hint'] = rg_template("hints/repo/anon_push.html", $repo_more);
	$repo_more['HTML:hints'] = rg_template_table("hints/list", $hints, $repo_more);

	$_repo_body .= rg_template("repo/source.html", $repo_more);

	rg_log("DEBUG: _subsubop=[$_subsubop]");
	if (strcmp($_subsubop, "tree") == 0) {
		$type = array_shift($paras);
		rg_log("DEBUG: tree: type=$type");
		if (strcmp($type, "blob") == 0) {
			// find hash of blob
			$_path = implode("/", $paras);
			$repo_more['path'] = "/" . $_path;
			$_tree = rg_git_ls_tree($ref, $_path);
			if ($_tree === FALSE) {
				$_repo_body .= "Invalid path!";
			} else {
				$blob = $_tree[0];
				$_hash = $_tree[0]['ref'];
				$c = rg_git_content($_hash);
				$_content = array(
					"hash" => $_hash,
					"HTML:content" => rg_template_list($c)
					);
				$blob = array_merge($blob, $_content);
				$_repo_body .= rg_template("repo/blob.html",
					array_merge($blob, $repo_more));
			}
		} else if (strcmp($type, "tree") == 0) {
			// find treeish of dir
			$_path = implode("/", $paras);
			$repo_more['path'] = "/" . $_path;
			$_tree = rg_git_ls_tree($ref, $_path);
			if ($_tree === FALSE) {
				$_repo_body .= "Invalid path!";
			} else {
				$_hash = $_tree[0]['ref'];
				$_tree = rg_git_ls_tree($_hash, "");
				$_repo_body .= rg_template_table("repo/tree", $_tree, $repo_more);
			}
		} else { // default is to show root tree
			$repo_more['path'] = "";
			$_tree = rg_git_ls_tree($ref, "");
			$_repo_body .= rg_template_table("repo/tree", $_tree, $repo_more);
		}
	} else { // log is default
		// show the log
		$log = rg_git_log($repo_path, 10, "", $ref, FALSE);
		if ($log === FALSE) {
			rg_log("Error generating log (" . rg_git_error() . ")!");
			$_repo_body .= rg_warning("Error generating log!");
			// TODO: rg_internal_error?
		} else if (empty($log)) {
			$_repo_body .= rg_template("repo/not_init.html", $repo_more);
		} else {
			//rg_log("DEBUG: log: " . print_r($log, TRUE));
			$_repo_body .= rg_git_log_template($log,
				"repo/log", $repo_more);
		}

		$type = array_shift($paras);
		//rg_log("DEBUG: log: type=$type");
		if (strcmp($type, "commit") == 0) {
			$commit = rg_git_reference($paras[0]);
			// TODO: check error code
			rg_log("Show diff for commit [$commit]");

			if (strstr($commit, "..")) {
				$t = explode("..", $commit);
				$first = $t[0];
				$second = $t[1];
			} else {
				$first = $commit . "~1";
				$second = $commit;
			}

			$log = rg_git_log($repo_path, 1, $first, $second, TRUE);
			if ($log === FALSE) {
				$_repo_body .= rg_warning("Could not generate log.");
				rg_internal_error("Could not generate log!");
			} else {
				// stats
				//rg_log("DEBUG: log[0]['files']: " . rg_array2string($log[0]['files']));
				$_r = rg_git_files_stats($log[0]['files'], "repo/fstat");
				//rg_log("DEBUG: files_stats: " . $_r);
				if ($_r === FALSE)
					rg_internal_error("Could not generate file stats"
						. " (" . rg_git_error() . ")!");
				else
					$_repo_body .= $_r;

				// diff
				//rg_log("DEBUG: log[0]['patches']: " . rg_array2string($log[0]['patches']));
				$_r = rg_git_diff($log[0]['patches'], "repo/diff.html");
				//rg_log("DEBUG: git_diff: " . $_r);
				if ($_r === FALSE)
					rg_internal_error("Could not generate patches"
						. " (" . rg_git_error() . ").");
				else
					$_repo_body .= $_r;
			}
		}
	}
	$_repo_right = rg_template("repo/source_r.html", $repo_more);
} else if (strcmp($_subop, "bug") == 0) {
	include($INC . "/user/repo/bug/main.php");
	$_repo_body .= $bug_body;
} else if (strcmp($_subop, "stats") == 0) {
	$_repo_body .= rg_template("repo/stats.html", $repo_more);
} else if (strcmp($_subop, "mr") == 0) {
	if (empty($paras)) {
		$_repo_body .= rg_template("repo/mrs.html", $repo_more);

		$r = rg_mr_load($db, $ri['repo_id'], 20);
		if ($r === FALSE) {
			$_repo_body .= "Error getting merge request list ("
				. rg_mr_error() . ").";
		} else {
			$_repo_body .= rg_template_table("repo/mr/list", $r, $repo_more);
		}
	} else {
		$mr = preg_replace('/[^0-9a-zA-Z_]/', '', array_shift($paras));
		do {
			$mri = rg_mr_load_one($db, $ri['repo_id'], $mr);
			if ($mri === FALSE) {
				$_repo_body .= "Error getting merge request (" . rg_mr_error() . ").";
				break;
			}

			$_log = rg_git_log($repo_path, 0, $mri['old_rev'], $mri['new_rev'], TRUE);
			if ($_log === FALSE) {
				$_repo_body .= rg_warning("Error generating patch.");
				break;
			}

			$mri['HTML:diff'] = rg_git_log_template($_log, "repo/log", $repo_more);
			$_repo_body .= rg_template("repo/mr/page.html", $mri, $repo_more);
		} while (0);
	}
}

$repo_more['HTML:repo_body'] = $_repo_body;
$repo_more['HTML:repo_right'] = $_repo_right;
$_repo_page = rg_template("repo/main.html", $repo_more);
?>
