<?php
error_reporting(E_ALL | E_STRICT);
ini_set("track_errors", "On");

// override the default
$rg_event_socket = "event.sock";

$INC = "../inc";
require_once($INC . "/init.inc.php");
require_once($INC . "/events.inc.php");
require_once($INC . "/sql.inc.php");
require_once($INC . "/struct.inc.php");
require_once($INC . "/fixes.inc.php");

rg_log_set_file("event.log");

$rg_sql_debug = 1;

// Defaults
$rg_admin_email = "rg@embedromix.ro";

$db = rg_sql_open("dbname=trg");
if ($db === FALSE) {
	rg_log("Cannot create a database (" . rg_sql_error() . ")!");
	exit(1);
}

$r = rg_state_set($db, "schema_version", "0");
if ($r !== TRUE) {
	echo "Cannot reset schema (" . rg_state_error() . ")!\n";
	exit(1);
}

$r = rg_sql_struct_update($db, RG_DROP_TABLES|RG_IGNORE_ERRORS);
if ($r !== TRUE) {
	rg_log("Cannot create struct (" . rg_sql_error() . ")!");
	exit(1);
}

$r = rg_fixes_update($db);
if ($r !== TRUE) {
	rg_log("Cannot aply fixes!");
	exit(1);
}

$r = rg_sql_struct_slaves_update($db);
if ($r !== TRUE) {
	rg_log("Cannot create slaves!");
	exit(1);
}

/*
 * This function will generate an array of sub-events
 */
function notif_new_repo($db, $ev)
{
	rg_log("notif_new_repo: ev=" . rg_array2string($ev));

	$ret = array();

	// Send mail to the owner
	$ret[] = array_merge($ev, array("category" => 1, "prio" => 1));

	// Send mail to admin
	$ret[] = array_merge($ev, array("category" => 2, "prio" => 1));

	// Other junk
	$ret[] = array_merge($ev, array("category" => 3, "prio" => 1));

	return $ret;
}

// Register an event function
$functions = array();
$functions[1] = "notif_new_repo";
$r = rg_event_register_functions($functions);
if ($r !== TRUE) {
	echo "Cannot register functions!\n";
	exit(1);
}

// defaults
$event = array(
	"prio" => 1,
	"category" => 1,
	"ri.repo_id" => 200
);

$r = rg_event_add($db, $event);
if ($r !== TRUE) {
	echo "Cannot add event (" . rg_event_error() . ")!\n";
	exit(1);
}

$notify_list = array();
$r = rg_event_process_queue($db, $notify_list);
if ($r === FALSE) {
	echo "Cannot process queue (" . rg_event_error() . ")!\n";
	exit(1);
}

rg_sql_close($db);

echo "event: OK!\n";
?>
