<?php
error_reporting(E_ALL | E_STRICT);
ini_set("track_errors", "On");

$INC = "../inc";
require_once($INC . "/init.inc.php");
require_once($INC . "/user.inc.php");
require_once($INC . "/sql.inc.php");
require_once($INC . "/struct.inc.php");
require_once($INC . "/fixes.inc.php");

$rg_sql_debug = 1;

rg_log_set_file("user.log");

// Defaults
$rg_repos = "ubase";
$rg_user_max_len = 20;
$rg_session_time = 3600;
$rg_user_allow = '/^[A-Za-z0-9_.-]*$/';
$rg_admin_email = "rg@embedromix.ro";
$rg_admin_name = "RocketGit Admin";
$_SERVER['HTTP_HOST'] = "fake.tld";


$db = rg_sql_open("dbname=trg");
if ($db === FALSE) {
	echo "Cannot create a database (" . rg_sql_error() . ")!\n";
	exit(1);
}

$r = rg_state_set($db, "schema_version", "0");
if ($r !== TRUE) {
	echo "Cannot reset schema (" . rg_state_error() . ")!\n";
	exit(1);
}

$r = rg_sql_struct_update($db, RG_DROP_TABLES);
if ($r !== TRUE) {
	echo "Cannot create structure (" . rg_sql_error() . ")!\n";
	exit(1);
}

$r = rg_fixes_update($db);
if ($r !== TRUE) {
	echo "Cannot apply fixes!\n";
	exit(1);
}

$r = rg_sql_struct_slaves_update($db);
if ($r !== TRUE) {
	rg_log("Cannot create slaves!");
	exit(1);
}

$r = rg_exec("rm -rf ubase");
if ($r['ok'] != 1) {
	echo "Cannot remove ubase dir (" . $r['errmsg'] . ")!\n";
	exit(1);
}

// add user
$_u['uid'] = 0;
$_u['realname'] = "userA real name";
$_u['username'] = "userA";
$_u['email'] = "rg@localhost";
$_u['pass'] = "pass1";
$_u['is_admin'] = 1;
$_u['disk_quota_mb'] = 100;
$_u['rights'] = "C";
$_u['session_time'] = 3600;
$_u['confirm_token'] = "";
$uid = rg_user_edit($db, $_u);
if ($uid === FALSE) {
	echo "Cannot add user (" . rg_user_error() . ")!\n";
	exit(1);
}

// Simulate event: link_by_name
$ev = array("uid" => $uid, "username" => $_u['username']);
$r = rg_user_link_by_name($db, $ev);
if ($r === FALSE) {
	echo "Cannot link by name (" . rg_user_error() . ")!\n";
	exit(1);
}

$v = rg_user_forgot_pass_mail_prepare($db, "rg@localhost");
if (empty($v['token'])) {
	print_r($v);
	echo "Error(rg_user_forgot_pass_mail): " . rg_user_error() . "!\n";
	exit(1);
}

$_ui = rg_user_info($db, $uid, "", "");
if ($_ui['exists'] != 1) {
	echo "Seems that user with $uid does not exists!\n";
	exit(1);
}
$pass = $_ui['pass'];

// edit user - empty pass
$_u['uid'] = $uid;
$_u['pass'] = "";
$r = rg_user_edit($db, $_u);
if ($r === FALSE) {
	echo "Cannot edit user with empty pass (" . rg_user_error() . ")!\n";
	exit(1);
}
// the pass should not be changed here
$_ui = rg_user_info($db, $uid, "", "");
if ($_ui['exists'] != 1) {
	echo "Seems that user with $uid does not exists!\n";
	exit(1);
}
if (strcmp($pass, $_ui['pass']) != 0) {
	echo "Password was changed!\n";
	exit(1);
}

// edit user - no empty pass
$_u['pass'] = "pass2";
$r = rg_user_edit($db, $_u);
if ($r === FALSE) {
	echo "Cannot edit user with not empty pass (" . rg_user_error() . ")!\n";
	exit(1);
}
// the pass should be changed here
$salt = $_ui['salt'];
$_ui = rg_user_info($db, $uid, "", "");
if ($_ui['exists'] != 1) {
	echo "Seems that user with $uid does not exists!\n";
	exit(1);
}
if (strcmp($salt, $_ui['salt']) == 0) {
	echo "Salt was not changed ($salt)!\n";
	exit(1);
}
if (strcmp($pass, $_ui['pass']) == 0) {
	echo "Password was not changed!\n";
	exit(1);
}

// change password
$r = rg_user_set_pass($db, $uid, "pass3");
if ($r !== TRUE) {
	echo "Cannot change pass!\n";
	exit(1);
}

// get token for e-mail forgot pass feature
$r = rg_user_forgot_pass_mail_prepare($db, "rg@localhost");
if (empty($r['token'])) {
	echo "Could not get token (" . rg_user_error() . ")!\n";
	exit(1);
}

$r = rg_user_forgot_pass_uid($db, $r['token']);
if ($r['ok'] != 1) {
	echo "Cannot find uid based on token (" . rg_user_error() . ")!\n";
	exit(1);
}

if ($r['uid'] != $uid) {
	echo "Token returned does not belong to the proper user!\n";
	exit(1);
}

// test rg_user_path
$x = sprintf("%08X", $uid);
$e = "ubase/by_id/" . substr($x, 0, 2) . "/" . substr($x, 2, 2)
	. "/" . substr($x, 4, 2) . "/" . substr($x, 6, 2) . "/" . $x;
$r = rg_user_path_by_id($uid);
if (strcmp($r, $e) != 0) {
	echo "path_by_id: e=[$e] != r=[$r]!\n";
	exit(1);
}

// test renames
$r = rg_user_rename($db, $_ui, "userA2");
if ($r === FALSE) {
	echo "Cannot rename user (" . rg_user_error() . ")!\n";
	exit(1);
}

rg_log("Testing fixes...");
// add user5
$_u['uid'] = 0;
$_u['realname'] = "user5 real name";
$_u['username'] = "user5";
$_u['email'] = "rg@localhost";
$_u['pass'] = "pass1";
$_u['is_admin'] = 1;
$_u['disk_quota_mb'] = 100;
$_u['rights'] = "C";
$_u['session_time'] = 3600;
$_u['confirm_token'] = "";
$uid5 = rg_user_edit($db, $_u);
if ($uid5 === FALSE) {
	echo "Cannot add user5 (" . rg_user_error() . ")!\n";
	exit(1);
}

$r = system("mkdir -p ubase/users/u/s/user5");
if ($r === FALSE) {
	echo "Cannot make fixes simulation dir ($php_errormsg)!\n";
	exit(1);
}
$r = rg_fixes_user_index_by_id($db);
if ($r === FALSE) {
	echo "cannot re-index by id!\n";
	exit(1);
}

echo "user: OK!\n";
?>
